@@ -5,7 +5,7 @@
 ;; Author: Protesilaos Stavrou <info@protesilaos.com>
 ;; Maintainer: Protesilaos Stavrou <info@protesilaos.com>
 ;; URL: https://github.com/protesilaos/consult-denote
-;; Version: 0.4.0
+;; Version: 0.4.2
 ;; Package-Requires: ((emacs "28.1") (denote "4.0.0") (consult "2.2"))
 
 ;; This file is NOT part of GNU Emacs.
@@ -130,9 +130,17 @@ Return the absolute path to the matching file."
                           (denote-directory-files
                            (or denote-file-prompt-use-files-matching-regexp files-matching-regexp)
                            :omit-current nil nil has-identifier)))
-         (prompt (format "%s in %s: " (or prompt-text "Select FILE") default-directory))
+         (prompt (if single-dir-p
+                     (format "%s: " (or prompt-text "Select FILE"))
+                   (format "%s in %s: "
+                           (or prompt-text "Select FILE")
+                           (propertize default-directory 'face 'denote-faces-prompt-current-name))))
          (input (consult--read
-                 (denote--completion-table 'file relative-files)
+                 (denote-get-completion-table
+                  relative-files
+                  '(category . file)
+                  '(group-function . denote-file-prompt-group)
+                  '(display-sort-function . denote-file-prompt-sort))
                  :state (consult--file-preview)
                  :require-match (unless no-require-match :require-match)
                  :history 'denote-file-history
@@ -150,6 +158,9 @@ Return the absolute path to the matching file."
     ;; appropriately.
     absolute-file))
 
+(declare-function denote-sequence-get-all-files "denote-sequence" (&optional files as-sequence-path-pairs))
+(declare-function denote-sequence-get-all-files "denote-sequence" (&optional files as-sequence-path-pairs))
+
 (defun consult-denote-sequence-file-prompt (&optional prompt-text files-with-sequences)
   "A Consult-powered equivalent of `denote-sequence-file-prompt'.
 
@@ -164,7 +175,11 @@ completion candidates.  Else use `denote-sequence-get-all-files'."
                               files))
             (prompt (format-prompt (or prompt-text "Select FILE with sequence") nil))
             (input (consult--read
-                    (denote--completion-table 'file relative-files)
+                    (denote-get-completion-table
+                     relative-files
+                     '(category . file)
+                     '(group-function . denote-file-prompt-group)
+                     '(display-sort-function . denote-file-prompt-sort))
                     :state (consult--file-preview)
                     :require-match nil
                     :history 'denote-sequence-file-history
@@ -177,12 +192,16 @@ completion candidates.  Else use `denote-sequence-get-all-files'."
 (defun consult-denote-select-linked-file-prompt (files &optional prompt-text)
   "Prompt for linked file among FILES and use optional PROMPT-TEXT."
   (let* ((single-dir-p (denote-has-single-denote-directory-p))
-         (file-names (if single-dir-p
-                         (mapcar #'denote-get-file-name-relative-to-denote-directory files)
-                       files))
+         (relative-files (if single-dir-p
+                             (mapcar #'denote-get-file-name-relative-to-denote-directory files)
+                           files))
          (prompt (format-prompt (or prompt-text "Find linked file") nil))
          (input (consult--read
-                 (denote--completion-table 'file file-names)
+                 (denote-get-completion-table
+                  relative-files
+                  '(category . file)
+                  '(group-function . denote-file-prompt-group)
+                  '(display-sort-function . denote-file-prompt-sort))
                  :state (consult--file-preview)
                  :require-match t
                  :history 'denote-link-find-file-history
@@ -191,17 +210,22 @@ completion candidates.  Else use `denote-sequence-get-all-files'."
         (expand-file-name input (car (denote-directories)))
       input)))
 
+(defvar denote-silo-extras-directory-history)
+(defvar denote-silo-extras-directories)
+
 (defun consult-denote-silo-directory-prompt ()
   "Like the `denote-silo-extras-directory-prompt' with Consult preview."
   (let ((default (car denote-silo-extras-directory-history)))
     (consult--read
-     (denote--completion-table 'file denote-silo-extras-directories)
+     (denote-get-completion-table denote-silo-extras-directories '(category . file))
      :state (consult--file-preview)
      :require-match t
      :prompt (format-prompt "Select a silo" default)
      :default default
      :history 'denote-silo-extras-directory-history)))
 
+(declare-function denote-org--get-outline "denote-org" (file))
+
 ;; FIXME 2024-07-03: We need a :state function that previews the
 ;; current line in the given buffer and then restores the window
 ;; configuration.
@@ -210,12 +234,12 @@ completion candidates.  Else use `denote-sequence-get-all-files'."
 FILE has the same meaning as in `denote-org-extras-outline-prompt'."
   (let ((current-file (or file buffer-file-name)))
     (consult--read
-     (denote--completion-table-no-sort 'imenu (denote-org-extras--get-outline current-file))
+     (denote-get-completion-table (denote-org--get-outline current-file) '(category . imenu))
      :state (lambda (_action candidate)
               (with-current-buffer (current-buffer)
                 (when-let* ((_ candidate)
                             (line (string-to-number (car (split-string candidate)))))
-                  (goto-line line (get-file-buffer current-file)))))
+                  (forward-line (- line 1)))))
      :prompt (format "Select heading inside `%s': " (propertize (file-name-nondirectory current-file) 'face 'denote-faces-prompt-current-name))
      :require-match t)))
 
@@ -227,7 +251,7 @@ FILE has the same meaning as in `denote-org-extras-outline-prompt'."
   (declare (interactive-only t))
   (interactive)
   (if (denote-has-single-denote-directory-p)
-      (funcall-interactively consult-denote-grep-command (denote-directory))
+      (funcall-interactively consult-denote-grep-command (car (denote-directories)))
     (let* ((directories (mapconcat #'identity (denote-directories) " "))
            (consult-grep-args `("grep" (consult--grep-exclude-args) "--null --line-buffered --color=never --ignore-case --with-filename --line-number -I -r" ,directories))
            (consult-ripgrep-args (format "rg --null --line-buffered --color=never --max-columns=1000 --path-separator --smart-case --no-heading --with-filename --line-number --search-zip %s" directories)))
@@ -239,7 +263,7 @@ FILE has the same meaning as in `denote-org-extras-outline-prompt'."
   (declare (interactive-only t))
   (interactive)
   (if (denote-has-single-denote-directory-p)
-      (funcall-interactively consult-denote-find-command (denote-directory))
+      (funcall-interactively consult-denote-find-command (car (denote-directories)))
     (let* ((directories (mapconcat #'identity (denote-directories) " "))
            (consult-find-args  (format "find %s -not ( -path */.[A-Za-z]* -prune )" directories)))
       (funcall-interactively consult-denote-find-command))))
@@ -267,7 +291,7 @@ FILE has the same meaning as in `denote-org-extras-outline-prompt'."
          (lambda (buffer)
            (when-let* ((file (buffer-file-name buffer))
                        ((buffer-live-p buffer))
-                       ((denote-filename-is-note-p file)))
+                       ((denote-file-has-denoted-filename-p file)))
              (buffer-name buffer)))
          (buffer-list))))
 
@@ -300,15 +324,19 @@ FILE has the same meaning as in `denote-org-extras-outline-prompt'."
 
 (with-eval-after-load 'denote-silo-extras
   (setq consult-denote-silo-source
-    `( :name "Denote silos"
-       :narrow ?L
-       :category file
-       :default t
-       :face consult-denote-directory
-       :history consult-denote-buffer-history
-       :action ,#'dired
-       :state ,#'consult--file-state
-       :items ,denote-silo-extras-directories)))
+        `( :name "Denote silos"
+           :narrow ?L
+           :category file
+           :default t
+           :face consult-denote-directory
+           :history consult-denote-buffer-history
+           :action ,#'dired
+           :state ,#'consult--file-state
+           :items ,denote-silo-extras-directories)))
+
+(declare-function denote-org-outline-prompt "denote-org" (&optional file))
+(declare-function denote-silo-directory-prompt "denote-silo" ())
+(declare-function denote-sequence-file-prompt "denote-sequence" (&optional prompt-text files-with-sequences))
 
 ;;;###autoload
 (define-minor-mode consult-denote-mode
@@ -323,15 +351,15 @@ FILE has the same meaning as in `denote-org-extras-outline-prompt'."
         (advice-add #'denote-file-prompt :override #'consult-denote-file-prompt)
         (advice-add #'denote-select-from-files-prompt :override #'consult-denote-select-linked-file-prompt)
         ;; See FIXME where this function is defined.
-        (advice-add #'denote-org-extras-outline-prompt :override #'consult-denote-outline-prompt)
-        (advice-add #'denote-silo-extras-directory-prompt :override #'consult-denote-silo-directory-prompt)
+        (advice-add #'denote-org-outline-prompt :override #'consult-denote-outline-prompt)
+        (advice-add #'denote-silo-directory-prompt :override #'consult-denote-silo-directory-prompt)
         (advice-add #'denote-sequence-file-prompt :override #'consult-denote-sequence-file-prompt))
     (dolist (source consult-denote-buffer-sources)
       (setq consult-buffer-sources (delq source consult-buffer-sources)))
     (advice-remove #'denote-file-prompt #'consult-denote-file-prompt)
     (advice-remove #'denote-select-from-files-prompt #'consult-denote-select-linked-file-prompt)
-    (advice-remove #'denote-org-extras-outline-prompt #'consult-denote-outline-prompt)
-    (advice-remove #'denote-silo-extras-directory-prompt #'consult-denote-silo-directory-prompt)
+    (advice-remove #'denote-org-outline-prompt #'consult-denote-outline-prompt)
+    (advice-remove #'denote-silo-directory-prompt #'consult-denote-silo-directory-prompt)
     (advice-remove #'denote-sequence-file-prompt #'consult-denote-sequence-file-prompt)))
 
 (provide 'consult-denote)
